@@ -13,7 +13,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import socket
 from pecan import expose, response, request, abort
 from pecan.rest import RestController
 import wsmeext.pecan as wsme_pecan
@@ -26,7 +25,9 @@ from libra.api.acl import get_limited_to_project
 from libra.api.model.validators import LBNodeResp, LBNodePost, NodeResp
 from libra.api.model.validators import LBNodePut
 from libra.api.library.gearman_client import submit_job
-from libra.api.library.exp import OverLimit
+from libra.api.library.exp import OverLimit, IPOutOfRange
+from libra.api.library.ip_filter import ipfilter
+from pecan import conf
 
 
 class NodesController(RestController):
@@ -35,8 +36,8 @@ class NodesController(RestController):
         self.lbid = lbid
         self.nodeid = nodeid
 
-    @expose('json')
-    def get(self, node_id=None):
+    @wsme_pecan.wsexpose(None)
+    def get(self):
         """List node(s) configured for the load balancer OR if
         node_id == None .. Retrieve the configuration of node {node_id} of
         loadbalancer {load_balancer_id}.
@@ -52,13 +53,9 @@ class NodesController(RestController):
         tenant_id = get_limited_to_project(request.headers)
 
         if not self.lbid:
-            response.status = 400
-            return dict(
-                message='Bad Request',
-                details='Load Balancer ID not supplied'
-            )
+            raise ClientSideError('Load Balancer ID not supplied')
         with db_session() as session:
-            if not node_id:
+            if not self.nodeid:
                 nodes = session.query(
                     Node.id, Node.address, Node.port, Node.status, Node.enabled
                 ).join(LoadBalancer.nodes).\
@@ -83,15 +80,12 @@ class NodesController(RestController):
                 ).join(LoadBalancer.nodes).\
                     filter(LoadBalancer.tenantid == tenant_id).\
                     filter(LoadBalancer.id == self.lbid).\
-                    filter(Node.id == node_id).\
+                    filter(Node.id == self.nodeid).\
                     first()
 
                 if node is None:
                     session.rollback()
-                    response.status = 400
-                    return dict(
-                        message='Bad Request', details='node not found'
-                    )
+                    raise ClientSideError('node not found')
 
                 node_response = node._asdict()
                 if node_response['enabled'] == 1:
@@ -137,8 +131,13 @@ class NodesController(RestController):
                     'Node {0} is missing a port'.format(node.address)
                 )
             try:
-                socket.inet_aton(node.address)
-            except socket.error:
+                node.address = ipfilter(node.address, conf.ip_filters)
+            except IPOutOfRange:
+                raise ClientSideError(
+                    'IP Address {0} is not allowed as a backend node'
+                    .format(node.address)
+                )
+            except:
                 raise ClientSideError(
                     'IP Address {0} not valid'.format(node.address)
                 )
@@ -226,11 +225,19 @@ class NodesController(RestController):
                 session.rollback()
                 raise ClientSideError('Node ID is not valid')
 
+            nodecount = session.query(Node).\
+                filter(Node.lbid == self.lbid).\
+                filter(Node.enabled == 1).count()
             if body.condition != Unset:
                 if body.condition == 'DISABLED':
                     node.enabled = 0
                 else:
                     node.enabled = 1
+            if nodecount <= 1:
+                session.rollback()
+                raise ClientSideError(
+                    "Cannot disable the last enabled node in a load balancer"
+                )
 
             lb.status = 'PENDING_UPDATE'
             device = session.query(
@@ -244,8 +251,8 @@ class NodesController(RestController):
             )
             return ''
 
-    @wsme_pecan.wsexpose(None)
-    def delete(self, status_code=202):
+    @wsme_pecan.wsexpose(None, status_code=202)
+    def delete(self):
         """Remove a node from the load balancer.
 
         :param load_balancer_id: id of lb
@@ -273,12 +280,13 @@ class NodesController(RestController):
                 raise ClientSideError("Load Balancer not found")
             load_balancer.status = 'PENDING_UPDATE'
             nodecount = session.query(Node).\
-                filter(Node.lbid == self.lbid).count()
+                filter(Node.lbid == self.lbid).\
+                filter(Node.enabled == 1).count()
             # Can't delete the last LB
             if nodecount <= 1:
                 session.rollback()
                 raise ClientSideError(
-                    "Cannot delete the last node in a load balancer"
+                    "Cannot delete the last enabled node in a load balancer"
                 )
             node = session.query(Node).\
                 filter(Node.lbid == self.lbid).\
